opts = optimset('display','off');

rng(13456465)

global Ta Prod EVI PAR NEE

load ../Data/antecedentPrecipPredictor.mat aPp
load allData.mat PAR Ta rBLUE rNIR rRED NEE rSWIR dy
load Tsoil.mat Tsoil

EVI  = 2.5*(rNIR - rRED) ./ (rNIR + 6*rRED - 7.5*rBLUE + 1);
LSWI = (rNIR - rSWIR) ./ (rNIR + rSWIR); 
Wscl = (1 + LSWI) ./ (1 + max(LSWI));
Pscl = .5*(1+LSWI);
Prod = Wscl .* Pscl;

vr    = cell(8,1);
vr{1} = 'PAR '; 
vr{2} = '{\itT}air';
vr{3} = '{\itr}BLUE';
vr{4} = '{\itr}NIR';
vr{5} = '{\itr}RED';
vr{6} = '{\itr}SWIR';
vr{7} = 'acp';
vr{8} = '{\itT}soil';
 
%        1   2   3    4     5     6   7    8
X   = [ PAR Ta rBLUE rNIR rRED rSWIR aPp Tsoil ];
y   = NEE;
Np  = length(X(1,:));   % number of aANN predictors 
Nt  = length(X(:,1));   % number of hourly observations
Nmc = 100;              % MC realization number
oh  = ones(1,Nmc);

if exist('JacobianHessian.mat')~=2

  Ta0   =   Ta;
  Prod0 = Prod;
  EVI0  =  EVI; 
  PAR0  =  PAR;
  NEE0  =  NEE;
  
  % lambda, Topt, PAR, alpha, beta, Tlow. Start from the HF values in
  % Mahadevan et al.'s Table 2
  p0  = [ .127 20 570 .271 .25 5 ];
  Prm = zeros(Nmc,6);
  Net =  cell(Nmc,1);
  
  nmc = 0;
    
  while nmc<Nmc

    k = randperm(Nt,3e4);

    Ta   =   Ta0(k);
    Prod = Prod0(k);
    EVI  =  EVI0(k); 
    PAR  =  PAR0(k);
    NEE  =  NEE0(k);

    [p,~,e] = fminsearch(@dNEE,p0,opts); 
    if e==1
      disp(p)
      yht = NEEnonLinReg(p,X);
      clear net
      net = feedforwardnet([6 5 4]);
      net.divideFcn = '';
      net.trainParam.showWindow = false;
      net = train(net,X(k,:)',y(k)');
      yht = [ yht' net(X') ];

      if max(abs(yht))<51 
	nmc        = nmc + 1;
	Prm(nmc,:) = p;
	Net{nmc}   = net;        disp(nmc)
      end
      
    end
    
  end
  
  save JacobianHessian.mat Prm Net

end

I     =  cell(4,1);
I{1}  = find( (dy>291 | dy<131) & PAR>900 );  
I{2}  = find( (dy>291 | dy<131) & PAR>  50 & PAR< 350);  
I{3}  = find(  dy>131 & dy<291  & PAR>900 );  
I{4}  = find(  dy>131 & dy<291  & PAR> 50 & PAR< 350 );  clear dy *scl
Ne    = length(I);        % number of doy and day--night epochs

dr    = cell(4,1);
dr{1} = 'winter high PAR';  dr{2} = 'winter low PAR';  
dr{3} = 'summer high PAR';  dr{4} = 'summer low PAR';  

load JacobianHessian.mat

if (exist('D1ANN')+exist('D1VPR'))<2
  
  % get numerical derivatives of both models
  D1ANN = zeros(Ne,Np,5);
  D1VPR = zeros(Ne,Np,5);
  for i = 1:Ne
    Xh = X(I{i},:);
    for j = 1:Np
      DN = zeros(1,Nmc);
      DM = zeros(1,Nmc);
      dx = sort(X(:,j));
      dx = diff(dx(round(Nt*[.05 .95])))*.05;
      x1 = median(Xh);   x1(j) = x1(j)+dx;
      x2 = median(Xh);   x2(j) = x2(j)-dx; 
      for k = 1:Nmc
	DN(k) = Net{k}(x1')' - Net{k}(x2')';
	DM(k) = NEEnonLinReg(Prm(k,:),x1) - NEEnonLinReg(Prm(k,:),x2);
      end
      DN = sort(DN);
      DM = sort(DM);
      k  = round(Nmc*[.005 .025 .5 .975 .995]);
      D1ANN(i,j,:) = DN(k);
      D1VPR(i,j,:) = DM(k); 
    end
  end

  save JacobianHessian.mat -append D1* ;  clear i j k DN DM x1 x2 dx
  
end

load JacobianHessian.mat D1*

ha = 'HorizontalAlignment';
nr = 'Normalized';
ec = 'EdgeColor';
lw = 'LineWidth';
un = 'Units';

clf

c  = brewermap(Np*2,'RdYlBu');
pn = 'abcd';

for i = 1:Ne
  
  axes('Position',[.08 .999-.225*i .35 .22])
  
  plot([0 19],[0 0],'k-',lw,.1); hold on
  plot(Np*[1 1]+.5,500*[-1 1],'k-',lw,.1)

  j = squeeze(D1ANN(i,:,3)); j = j(:);
  k = squeeze(D1VPR(i,:,3)); k = k(:); j = [ j ; k ];
  dy =  max(j)*1.05 - min(j)*1.05;
  
  for j = 1:Np
    fill(j   +.44*[-1 -1 1 1],[0 D1ANN(i,j,3)*[1 1] 0],c(     j  ,:),ec,'w')
    fill(j+Np+.44*[-1 -1 1 1],[0 D1VPR(i,j,3)*[1 1] 0],c(2*Np-j+1,:),ec,'w')
    if i==Ne   
      text(j,-6.6,vr{j},'Rotation',48,ha,'r')
      if j<(Np-1)
	text(Np+j,-6.6,vr{j},'Rotation',48,ha,'r')
      end
    end
    if abs(D1ANN(i,j,3))>0
      t = '';
      if sign(D1ANN(i,j,2))==sign(D1ANN(i,j,4)); t = [ t '*']; end
      if sign(D1ANN(i,j,1))==sign(D1ANN(i,j,5)); t = [ t '*']; end
      if D1ANN(i,j,3)>0; text(j,-.5,t,ha,'c');
      else;  text(j,.1,t,ha,'c'); end
    end
  end
  plot([0 19],[0 0],'k-',lw,.1); hold on
  axis([ .3 2*Np-1.5 -6.3 3.8 ])
  set(gca,'XTick',1:2*Np,'XTickLabel',[],'Box','off')
  if i<Ne; set(gca,'XTickLabel',[]);   end
  if i==1
    text(.33,1.03,'aANN',un,nr,ha,'c')
    text(.82,1.03,'VPRM (all **)',un,nr,ha,'c')
    t = '\DeltaNEE per \Delta(input variable) = 10% ';
    t = [ t 'of total epochal range'];
    text(-.12,.35,t,un,nr,ha,'r','Rotation',90)
  end
  text(.01,.94,[ '{\bf' pn(i) '}'],un,nr)
  text(.99,.18,dr{i}(1: 6 ),un,nr,ha,'r')
  text(.99,.08,dr{i}(8:end),un,nr,ha,'r')
end

set(gcf,'PaperPosition','default')
print -dpdf -r350 -painters JacobianHessian.pdf

