load ../Matlab/allData NEE t dy % hourly HF tower data
tN = datenum([floor(t)*[1 0] dy+(1/48)]) - datenum(1999,1,1);

load harvardForestDailyPrecip.mat tp HFp % daily precip data from near HF

% make the HFp precipitation record pseudo-hourly resolution, 
% for conveneience of lag calculating
HFp = (HFp*ones(1,24))';
HFp = HFp(:)/24; % make continuous and convert to mm/hr

i   = (tp(:,1)*ones(1,24))'; tph =       i(:)  ; % yr
i   = (tp(:,2)*ones(1,24))'; tph = [ tph i(:) ]; % mn
i   = (tp(:,3)*ones(1,24))'; tph = [ tph i(:) ]; % dy
i   = [.5:24]'*ones(1,length(HFp)/24);   
i   = i(:);
tph = [ tph i*[1 0 0] ]; % add the hr/min/sec columns
tp  = datenum(tph) - datenum(1999,1,1);

% smooth a bit the HF pseudo-hourly precip
i = conv(HFp,ones(1,25)/25); 
HFp(5:end-4) = i(17:end-16);

% make the last time point in both datasets the same
i = 1:max(find(tp==tN(end)));
tp  = tp(i);
HFp = HFp(i);   clear i tph dy t

% cummulative antecedent precip in 8-hr increments
aP  = nan*zeros(length(tN),125*3);
NaP = nan*zeros(length(tN),125*3);
i0  = find( tp==tN(end) );
for i = 1:length(tN) % loop on time points
  for j = 4:8:125*24 % loop on accumulation hours
    k = find( tp<tN(i) & 24*(tN(i)-tp)<(j+4) );
     aP(i,(j+4)/8) = sum(HFp(k))/j; % cumulative mm over j preceding hours
  end
  disp(i)
end

Capn = [];
N    = [];
for i = 1:125*3
  k = find(~isnan(aP(:,i)));
  Capn = [ Capn cor4vects(NEE(k),aP(k,i)) ];
  N = [ N length(k) ];
end

clf

tau = [4:8:125*24]/24;

axes('Position',[.2 .5 .4 .4])
plot(tau,Capn,'.r-','MarkerSize',2,'LineWidth',.1); hold on
xlabel('\tau, days of cumulative antecedent precipitation')
ylabel('[NEE,\int_{\itt-\tau}^{\itt} precip.{\it(t) dt}] correlation')
text(.99,.94,'{\itN} \approx 44.9{\itk} observations', ...
    'Units','normalized','HorizontalAlignment','r')
axis([-2 max(tau)+2 min(Capn)-.002 max(Capn)+.003])
[i,j] = min(Capn);
tj = tau(j);
plot([tj tj],[-1 i+.02],'k-')
text(tj,i+.025,[ 'c_{\tau = ' int2str(tj) '} \approx ' sprintf('%4.2f',i)])

set(gcf,'PaperPosition','default')
print -dpdf -r350 -painters bestPrecipLag.pdf

% OK, so tau = 63 days (1515 hrs to be exact) is the winner. 
% So - make a predictor timeseries of it

aPp = aP(:,j);
tap = tN;

save antecedentPrecipPredictor.mat aPp tap
